@@ -1,7 +1,7 @@
 /***************************************************************************
  *
  * Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
- * 2010 BalaBit IT Ltd, Budapest, Hungary
+ * 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -22,8 +22,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: dimhash.c,v 1.15 2003/04/11 14:44:59 sasa Exp $
- *
  * Author  : SaSa
  * Auditor :
  * Last audited version:
@@ -378,7 +376,7 @@ gpointer
 z_dim_hash_table_search(ZDimHashTable *self, guint num, gchar **keys)
 {
   gchar *save_keys[DIMHASH_MAX_KEYNUM];
-  gpointer *ret;
+  gpointer *ret = NULL;
   guint i;
 
   z_enter();  
