@@ -547,7 +547,7 @@ z_ifmon_add_iface(const gchar *msg, gsize msg_len)
   /* interface was renamed, old_iface refers to the old name */
   old_iface_changed = !new && if_name && strcmp(info->name, if_name) != 0;
   /* interface with the new/current name was changed */
-  new_iface_changed = new || ((info->flags & IFF_UP) != (if_flags & IFF_UP)) || old_iface_changed;
+  new_iface_changed = new || ((info->flags & IFF_UP) != (guint32) (if_flags & IFF_UP)) || old_iface_changed;
 
   /* send notification */
   if (old_iface_changed)
@@ -643,7 +643,7 @@ z_ifmon_change_iface_addr(const gchar *msg, gsize msg_len)
   gint nl_type = ((struct nlmsghdr *) msg)->nlmsg_type;
   guint ifa_index;
   guint ifa_family;
-  const gchar *ifa_addr;
+  const gchar *ifa_addr = NULL;
   struct in_addr *ina;
   ZIfaceInfo *info;
   gchar buf[32];
