@@ -69,30 +69,14 @@ static PyMethodDef py_zorp_stream_methods[] =
 
 PyTypeObject z_policy_stream_type = 
 {
-  PyObject_HEAD_INIT(&PyType_Type)
-  0,
-  "ZPolicyStream",
-  sizeof(ZPolicyStream),
-  0,
-  (destructor) z_policy_stream_destroy,
-  0,
-  (getattrfunc) z_policy_stream_getattr,
-  (setattrfunc) z_policy_stream_setattr,
-  0,
-  (reprfunc) z_policy_stream_repr,
-  0,
-  0,
-  0,
-  0,
-  0,
-  0,
-  0,
-  0,
-  0,
-  0,
-  "ZPolicyStream class for Zorp",
-  0, 0, 0, 0,
-  Z_PYTYPE_TRAILER
+  PyVarObject_HEAD_INIT(&PyType_Type, 0)
+  .tp_name = "ZPolicyStream",
+  .tp_basicsize = sizeof(ZPolicyStream),
+  .tp_dealloc = (destructor) z_policy_stream_destroy,
+  .tp_getattr = (getattrfunc) z_policy_stream_getattr,
+  .tp_setattr = (setattrfunc) z_policy_stream_setattr,
+  .tp_repr = (reprfunc) z_policy_stream_repr,
+  .tp_doc = "ZPolicyStream class for Zorp",
 };
 
 /**
@@ -411,9 +395,8 @@ z_policy_stream_write(PyObject *o, PyObject *args)
       PyErr_SetString(PyExc_IOError, "I/O error writing stream.");
       return NULL;
     }
-  
-  Py_XINCREF(Py_None);
-  return Py_None;
+
+  z_return(z_policy_none_ref());
 }
 
 
@@ -430,8 +413,8 @@ z_policy_stream_close(PyObject *o, PyObject *args G_GNUC_UNUSED)
   ZPolicyStream *self = (ZPolicyStream *) o;
 
   z_stream_close(self->stream, NULL);
-  Py_XINCREF(Py_None);
-  return Py_None;
+
+  z_return(z_policy_none_ref());
 }
 
 /**
