@@ -1,13 +1,16 @@
 #ifndef ZORP_CONNECTION_H_INCLUDED
 #define ZORP_CONNECTION_H_INCLUDED
 
+#include <zorp/zorp.h>
 #include <zorp/stream.h>
 #include <zorp/sockaddr.h>
 
-#define ZD_PROTO_TCP  1
-#define ZD_PROTO_UDP  2
-#define ZD_PROTO_TCP6 3
-#define ZD_PROTO_UDP6 4
+enum
+{
+  ZD_PROTO_AUTO = 0,
+  ZD_PROTO_TCP = 1,
+  ZD_PROTO_UDP = 2,
+};
 
 typedef struct _ZConnection
 {
@@ -16,7 +19,7 @@ typedef struct _ZConnection
   ZSockAddr *remote; /* the peer's address */
   ZSockAddr *local;  /* the explicit local address (no wildcard port spec) */
   ZSockAddr *dest;   /* the original destination of the client */
-  ZSockAddr *bound;  /* the requested bind address */
+  struct _ZDispatchBind *dispatch_bind;
 } ZConnection;
 
 ZConnection *z_connection_new(void);
