@@ -2,13 +2,13 @@
  * Transparent proxy support for Linux/iptables
  *
  * Copyright (c) 2002 BalaBit IT Ltd.
- * Author: Bal<E1>zs Scheidler 
+ * Author: Balzs Scheidler 
  *
  * This code is under GPLv2.
  */
 
-#ifndef _IP_TPROXY_H
-#define _IP_TPROXY_H
+#ifndef _IP_TPROXY_OLD_H
+#define _IP_TPROXY_OLD_H
 
 #ifdef __KERNEL__
 #include <linux/types.h>
@@ -25,6 +25,8 @@
 #define IP_TPROXY_UNASSIGN 21
 #define IP_TPROXY_QUERY    22
 #define IP_TPROXY_FLAGS    23
+#define IP_TPROXY_ALLOC    24
+#define IP_TPROXY_CONNECT  25
 
 /* bitfields in IP_TPROXY_FLAGS */
 #define ITP_CONNECT     0x00000001
@@ -38,8 +40,8 @@
 
 /* structure passed to setsockopt(SOL_IP, IP_TPROXY) */
 struct in_tproxy {
-        struct in_addr itp_faddr;
-        u_int16_t itp_fport;
+	struct in_addr itp_faddr;
+	u_int16_t itp_fport;
 };
 
 #endif
