@@ -3,6 +3,14 @@
 
 #include <zorp/proxy/transfer2.h>
 
+enum
+{
+  DOT_DW_PREAMBLE = 0,
+  DOT_DW_DATA     = 1,
+  DOT_DW_DATA_LF  = 2,
+  DOT_DW_DATA_DOT = 3,
+};
+
 typedef struct _ZDotTransfer
 {
   ZTransfer2 super;
@@ -19,7 +27,6 @@ z_dot_transfer_new(ZClass *class,
                    ZProxy *owner,
                    ZPoll *poll,
                    ZStream *client, ZStream *server,
-                   ZStackedProxy *stacked,
                    gsize buffer_size,
                    glong timeout,
                    gulong flags,
