@@ -317,7 +317,7 @@ extern ZClass FtpProxy__class;
 #define MSG_NT_SERVER_HANDSHAKE_FAILED 37
 #define MSG_NT_SERVER_CERT_INVALID_SUBJECT 38
 #define MSG_NT_SERVER_AUTH_REJECT      39
-#define MSG_NT_SERVER_PBSZ_REJECT      40
+#define MSG_NT_SERVER_PBSZV_REJECT      40
 #define MSG_NT_SERVER_PROT_REJECT      41
 #define MSG_USER_INBAND_INFO_INVALID   42
 
