@@ -1,7 +1,7 @@
 /***************************************************************************
  *
  * Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
- * 2010 BalaBit IT Ltd, Budapest, Hungary
+ * 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -22,8 +22,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: httphdr.c,v 1.19 2004/07/26 11:45:57 bazsi Exp $
- * 
  * Author: Balazs Scheidler <bazsi@balabit.hu>
  * Auditor: 
  * Last audited version: 
@@ -154,7 +152,9 @@ http_add_header(HttpHeaders *hdrs, gchar *name, gint name_len, gchar *value, gin
         }
       else
         {
-          z_log(NULL, HTTP_VIOLATION, 3, "Possible smuggle attack, removing header duplication; header='%.*s', value='%.*s', prev_value='%.*s'", name_len, name, value_len, value, orig->value->len, orig->value->str);
+          z_log(NULL, HTTP_VIOLATION, 3,
+              "Possible smuggle attack, removing header duplication; header='%.*s', value='%.*s', prev_value='%.*s'",
+              name_len, name, value_len, value, (gint) orig->value->len, orig->value->str);
         }
     }
   return h;
@@ -507,7 +507,8 @@ http_filter_headers(HttpProxy *self, guint side, HttpHeaderFilter filter)
               if (info->max_len >= 0 && h->value->len > (guint) info->max_len)
                 {
                   z_proxy_log(self, HTTP_VIOLATION, 3,
-                              "Header failed strict checking, value too long; header='%s', value='%s', length='%d', max_length='%d'",
+                              "Header failed strict checking, value too long; "
+                              "header='%s', value='%s', length='%" G_GSIZE_FORMAT "', max_length='%" G_GSIZE_FORMAT "'",
                               h->name->str, h->value->str, h->value->len, info->max_len);
                   action = self->strict_header_checking_action;
                   goto exit_check;
@@ -515,7 +516,8 @@ http_filter_headers(HttpProxy *self, guint side, HttpHeaderFilter filter)
               if (!http_check_header_charset(self, h->value->str, info->flags, &reason))
                 {
                   z_proxy_log(self, HTTP_VIOLATION, 3,
-                              "Header failed strict checking, it contains invalid characters; header='%s', value='%s', reason='%s'",
+                              "Header failed strict checking, it contains invalid characters; "
+                              "header='%s', value='%s', reason='%s'",
                               h->name->str, h->value->str, reason);
                   action = self->strict_header_checking_action;
                   goto exit_check;
