@@ -1,7 +1,7 @@
 ############################################################################
 ##
 ## Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-## 2010 BalaBit IT Ltd, Budapest, Hungary
+## 2010, 2011 BalaBit IT Ltd, Budapest, Hungary
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -35,8 +35,9 @@
 """
 
 from Zorp import *
-from SockAddr import SockAddrInet, inet_ntoa, inet_aton, htonl, ntohl
+from SockAddr import SockAddrInet, inet_ntoa, inet_aton
 from string import split, atoi
+from socket import htonl, ntohl
 try:
 	from SockAddr import inet_pton, inet_ntop
 except ImportError:
