@@ -2,7 +2,7 @@ from Zorp.Core import *
 from Zorp.Plug import *
 
 from Zorp.Session import MasterSession
-from Zorp.NAT import getNATPolicy, NAT_SNAT, NAT_DNAT
+from Zorp.NAT import NAT_SNAT, NAT_DNAT
 from Zorp.Zorp import quit
 from traceback import *
 
@@ -13,23 +13,24 @@ def testcase(nat, session, addrs, type, expected_result):
 	
 	if not res.equal(expected_result):
 		print 'invalid result res=%s, expected_result=%s' % (res.format(), expected_result.format())
-		raise 'test error'
+		raise ValueError
 
 def zorp():
 	try:
 		s = MasterSession()
 		s.setService(Service("s1", None))
 	
-		NATPolicy('test', GeneralNAT(
-				[(InetDomain('10.0.0.0/8'), InetDomain('20.0.0.0/8')),
-				 (InetDomain('11.0.0.0/8'), InetDomain('192.168.0.0/24')),
+		nat = NATPolicy('test', GeneralNAT(
+				[(InetSubnet('0.0.0.0/32'), InetSubnet('10.0.0.0/8'), InetSubnet('20.0.0.0/8')),
+				 (InetSubnet('0.0.0.0/32'), InetSubnet('11.0.0.0/8'), InetSubnet('192.168.0.0/24')),
+                                 (Inet6Subnet('::/128'), Inet6Subnet('1200::/8'), Inet6Subnet('2300::/8')),
 				]))
-		nat = getNATPolicy('test')
 		
 		testcase(nat, s, (None, SockAddrInet('10.0.0.1', 8888)), NAT_DNAT, SockAddrInet('20.0.0.1', 8888))
 		testcase(nat, s, (None, SockAddrInet('11.0.0.0', 8888)), NAT_DNAT, SockAddrInet('192.168.0.0', 8888))
 		testcase(nat, s, (None, SockAddrInet('11.0.1.1', 8888)), NAT_DNAT, SockAddrInet('192.168.0.1', 8888))
 		testcase(nat, s, (None, SockAddrInet('11.255.255.255', 8888)), NAT_DNAT, SockAddrInet('192.168.0.255', 8888))
+                testcase(nat, s, (None, SockAddrInet6('1234::', 8888)), NAT_DNAT, SockAddrInet6('2334::', 8888))
 	except Exception, e:
 		print_exc()
 		quit(1)
